varying vec4 vDiffuse;
varying vec4 vAmbient;

varying vec3 vNormalEyeSpace;
varying vec3 vLightDir;
varying vec3 vHalfVector;

uniform sampler2D tex;

vec4 doLighting(
	vec4 clAmbient, vec4 clDiffuse,	
	vec3 lDir, gl_LightSourceParameters light, vec3 halfVector, vec3 normal ) {
	
	/* compute the cos of the angle between the normal and lights direction. 
	The light is directional so the direction is constant for every vertex.
	Since these two are normalized the cosine is the dot product. We also 
	need to clamp the result to the [0,1] range. */
	float NdotL = max( dot( normal, lDir ), 0.0 );
	
	vec4 specular = vec4( 0, 0, 0, 0 );
	vec4 diffuse = vec4( 0, 0, 0, 0 );
	if (NdotL > 0.0) {
		/* compute the DIFFUSE term */
		diffuse = NdotL * clDiffuse;
	
		/* compute the SPECULAR term */
		/* - Blinn-Phong model (faster simplification of the Blinn model) */
		// normalize the half-vector, and then compute the 
		// cosine (dot product) with the normal
		float NdotHV = max( dot( normal, halfVector ), 0.0 );
		specular = gl_FrontMaterial.specular * light.specular * 
				pow( NdotHV, gl_FrontMaterial.shininess );
	}
	
	return vec4( (clAmbient + diffuse + specular).rgb, clDiffuse.a );
}

vec4 combineCT0( vec4 color ) {
	// requirements from vertex shader:
	// uniform sampler2D tex;

	// does filtering, mipmapping, ..... as configured in the program
	vec4 texel = texture2D( tex, gl_TexCoord[0].st );
	
	// GL_REPLACE
	//return texel;
	//return color;
	
	// GL_MODULATE
//	return texel * color;
	
	// GL_DECAL
	// cc*(1.-ta) +  tc*ta
//	return vec4( mix( color.rgb, texel.rgb, texel.a ), color.a );
	// tc*(1.-ta) +  cc*ca
	return vec4( mix( texel.rgb, color.rgb, color.a ), texel.a );

	// GL_BLEND
//	return vec4( mix( color.rgb, texel.rgb, texel.rbg ), color.a * texel.a );

	// GL_ADD
//	vec4 c;
//	c.rgb = color.rgb + texel.rgb;
//	c.a = color.a * texel.a;
//	return clamp( c, 0.0, 1.0 );
}

void main()
{
	vec4 lightedMaterial = doLighting(
		vAmbient,
		vDiffuse,
		vLightDir,
		gl_LightSource[0],
		normalize( vHalfVector ),
		normalize( vNormalEyeSpace ) );
	
//	gl_FragColor = lightedMaterial;
//	gl_FragColor = texture2D( tex, gl_TexCoord[0].st );
	gl_FragColor = combineCT0( lightedMaterial );
}
