varying vec4 vDiffuse;
varying vec4 vAmbient;

varying vec3 vNormalEyeSpace;
varying vec3 vLightDir;
varying vec3 vHalfVector;

uniform sampler2D tex;
uniform sampler2D shadowMap;

varying vec4 ShadowCoord;
float Epsilon = 0.01;
float shadowMapSize = 512.;

bool isHiddenBySM( float x, float y ) {
	//float depth = shadow2DProj( shadowMap,
	//				ShadowCoord + vec4( x, y, 0, 0 ) * Epsilon ).x;	
	
	// manual....
	vec2 coords = ShadowCoord.st + vec2( x, y ) * 1./shadowMapSize;
	if( (coords.s < 0.) || (coords.s > 1.) || (coords.t < 0.) || (coords.t > 1.) )
		return false;
	
	float depth = texture2D( shadowMap, coords ).z;
//	float depth = texture2DProj( shadowMap, ShadowCoord.stq ).z;

	float epsilon = .002;
	return ( ShadowCoord.z - depth ) > epsilon;
}

float isHiddenBySMSampled( float shadowFactor ) {
	float lightFactor = ( 1. - shadowFactor ) / 4.;
	float sum = shadowFactor;

	// check 4 samples
	if( ! isHiddenBySM( -.5, -.5 ) ) sum += lightFactor;
	if( ! isHiddenBySM( .5, -.5 ) ) sum += lightFactor;
	if( ! isHiddenBySM( -.5, .5 ) ) sum += lightFactor;
	if( ! isHiddenBySM( .5, .5 ) ) sum += lightFactor;

	return sum;
}


vec4 doLighting(
	vec4 clAmbient, vec4 clDiffuse,	
	vec3 lDir, gl_LightSourceParameters light, vec3 halfVector, vec3 normal ) {
	
	/* compute the cos of the angle between the normal and lights direction. 
	The light is directional so the direction is constant for every vertex.
	Since these two are normalized the cosine is the dot product. We also 
	need to clamp the result to the [0,1] range. */
	float NdotL = max( dot( normal, lDir ), 0.0 );
	
	float shadowFactor = 1.;
	if (ShadowCoord.x >= 0.0 && ShadowCoord.y >= 0.0 && ShadowCoord.x <= 1.0 && ShadowCoord.y <= 1.0)
		shadowFactor = isHiddenBySMSampled( .3 );
	
	vec4 specular = vec4( 0, 0, 0, 0 );
	vec4 diffuse = vec4( 0, 0, 0, 0 );
	if (NdotL > 0.0) {
		/* compute the DIFFUSE term */
		diffuse = NdotL * clDiffuse * shadowFactor;
	
		if (shadowFactor == 1.) {
			/* compute the SPECULAR term */
			/* - Blinn-Phong model (faster simplification of the Blinn model) */
			// normalize the half-vector, and then compute the 
			// cosine (dot product) with the normal
			float NdotHV = max( dot( normal, halfVector ), 0.0 );
			specular = gl_FrontMaterial.specular * light.specular * 
				pow( NdotHV, gl_FrontMaterial.shininess );
		}
	}
	
	return vec4( (clAmbient + diffuse + specular).rgb, clDiffuse.a );
}

vec4 combineCT0( vec4 color ) {
	// requirements from vertex shader:
	// uniform sampler2D tex;

	// does filtering, mipmapping, ..... as configured in the program
	vec4 texel = texture2D( tex, gl_TexCoord[0].st );
	
	// GL_REPLACE
	//return texel;
	//return color;
	
	// GL_MODULATE
//	return texel * color;
	
	// GL_DECAL
	// cc*(1.-ta) +  tc*ta
//	return vec4( mix( color.rgb, texel.rgb, texel.a ), color.a );
	// tc*(1.-ta) +  cc*ca
	return vec4( mix( texel.rgb, color.rgb, color.a ), texel.a );

	// GL_BLEND
//	return vec4( mix( color.rgb, texel.rgb, texel.rbg ), color.a * texel.a );

	// GL_ADD
//	vec4 c;
//	c.rgb = color.rgb + texel.rgb;
//	c.a = color.a * texel.a;
//	return clamp( c, 0.0, 1.0 );
}

void main()
{
	vec3 normal = normalize( vNormalEyeSpace );
	float NdotL = max( dot( normal, vLightDir ), 0.0 );

	vec4 dif = combineCT0( vDiffuse );
	vec4 col = doLighting(
		vAmbient,
		dif,
		vLightDir,
		gl_LightSource[0],
		normalize( vHalfVector ),
		normal );
	
//	if ( (NdotL > 0.0) &&
//	(ShadowCoord.x >= 0.0 && ShadowCoord.y >= 0.0 && ShadowCoord.x <= 1.0 && ShadowCoord.y <= 1.0) )
//		gl_FragColor = vec4( isHiddenBySMSampled( .3 ) * col.rgb, col.a );
//	else gl_FragColor = col;

	gl_FragColor = col;
}